import sys

from ctypes import *
lp = CDLL("/usr/local/lib/libpass-python.so")

lp.pydpapi_init()


def mkphony(pd):
    '''
    Create a phony on the same volume as pd
    
    pd: the file descriptor of a file located on a Lasagna volume
    '''
    assert (type(pd) == type(0)) or (str(type(pd)).find("provenance.wrapper") >= 0)
    return lp.mkphony(pd)


def addxref(pd,key,xref_pd):
    '''
    Add a cross-reference from pd to xref_pd with edge label key
    
    pd: the file descriptor of a source node
    key: the edge label (usually "INPUT")
    xref_pd: the file descriptor of a target node
    '''
    assert type(key) == type('')
    retlen = lp.addxref(c_int(pd), key, c_int(xref_pd))
    return retlen


def addstr(pd, key, value):
    '''
    Add a string attribute
    
    pd: the file descriptor of a node
    key: the attribute name (such as "NAME" or "TYPE")
    value: the attribute value
    '''
    assert type(key) == type('')
    assert type(value) == type('')    
    retlen = lp.addstr(c_int(pd), key, value)
    return retlen


def freeze(pd):
    '''
    Freeze the current version of a node
    
    pd: the file descriptor of a node
    '''
    retlen = lp.freeze(c_int(pd))
    return retlen


def syncphony(pd):
    '''
    Force the provenance of the given phony and all of its ancestors to disk
    
    pd: the file descriptor of a node
    '''
    retlen = lp.sync_phony(c_int(pd))
    return retlen


def closephony(pd):
    '''
    Close the handle of a phony
    
    pd: the file descriptor of a node
    '''
    retlen = lp.close_phony(c_int(pd))
    return retlen


def revivephony(pd, pnode, version):
    '''
    Revive a phony and return its file descriptor
    
    pd: the file descriptor of a file located on a Lasagna volume
    pnode: the pnode number
    version: the version of the phony
    '''
    retlen = lp.revive_phony(c_int(pd), c_int(pnode), c_int(version))
    return retlen


def getpnode(pd):
    '''
    Return the pnode number of a given file descriptor
    
    pd: the file descriptor
    '''
    return lp.get_pnode(c_int(pd))


def getversion(pd):
    '''
    Return the version number of a given file descriptor
    
    pd: the file descriptor
    '''
    return lp.get_version(c_int(pd))

