/*
 * Copyright 2006, 2007
 *	The President and Fellows of Harvard College.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE UNIVERSITY AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE UNIVERSITY OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>
#include <assert.h>
#include "twig.h"
#include "twig_file.h"

/**
 * Read and pretty print a twig file.
 *
 * Takes a twig format binary file and prints it as text.
 */

static void usage(char *name)
{
   fprintf(stderr,
           "usage: %s [-h] <files>\n"
           "   -h             usage\n",
           name );
}

int
main(int argc, char *argv[])
{
   const char *filename = NULL;
   struct twig_file  *file;
   struct twig_rec *rec = NULL;
   int index;

   //
   // Process arguments
   //

   int ch;
   while ( -1 != (ch = getopt(argc, argv, "h")) ) {
      switch ( ch ) {
         case 'h':
         default:
            usage(argv[0]);
            return 0;
      }
   }

   if (optind == argc) {
      usage(argv[0]);
      return 0;
   }


   //
   // Process all files in the argument list
   //
   
   for (index = optind; index < argc; index++) {

      filename = argv[index];

      //
      // Open the file
      //

      file = twig_open(filename, TWIG_RDONLY);
      if (file == NULL) {
         fprintf(stderr, "%s: Could not open %s\n", argv[0], filename);
         return 1;
      }

      printf( "file: %s\n", filename );

      //
      // Read and process the records
      //

      while (EOF != twig_read(file, &rec) ) {
         twig_print_rec(rec);

         rec = NULL;
//      free(rec), rec = NULL;
      }

      twig_close(file);
   }
   
   return 0;
}
