/*
 * Copyright 2006, 2007, 2010
 *	The President and Fellows of Harvard College.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE UNIVERSITY AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE UNIVERSITY OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <errno.h>
#include <ctype.h>
#include <getopt.h>
#include <assert.h>
#include <sys/time.h>
#include <utime.h>
#include <unistd.h>
#include <sys/wait.h>
#include <sys/types.h>
#include <fcntl.h>
#include <sys/stat.h>

#include <db.h>
#include "debug.h"

#include "twig.h"
#include "schema.h"
#include "wdb.h"
#include "ztfb.h"
#include <time.h>
#include <linux/types.h>
#include <asm/types.h>


static void dump_something(int ch);
static void dump_provdb(void);
static void dump_tnum2tok(void);
static void dump_tok2tnum(void);
static void dump_arg2p(void);
static void dump_env2p(void);
static void dump_p2i(void);
static void dump_i2p(void);
static void dump_name(void);
static void dump_child(void);
static void dump_parent(void);
static void dump_guize(void);
static void dump_ky2(void);


static void rebuild(void);
static void guize(void);
static void jiance(void);
static int find(char * ,char *);
static int insert_detect(pnode_t ,pnode_t);
static version_t get_file_version(pnode_t pnum);
static int get_file_pv(char * file_name, struct pnode_version * pv);
static pnode_version execute(struct pnode_version pv);
static int execute1(struct pnode_version pv);
static int file_access(struct pnode_version pv);
static int handling_state(struct pnode_version pv);
static int recover(int num);
static int analysis(struct pnode_version pv);
static int proc_rebuild(pnode_t pnum);
static char * get_string(pnode_t pnum, const char * attr);
static char ** get_multistring(pnode_t pnum, const char * attr);

static void print_attrflags(uint8_t attrflags);
int                         state[1000];
struct pnode_version  bak[1000];
int j=0;

/*************************add-by-dt**********************/
clock_t start,duration; //calc the program run time

struct node_pro{
	char *name;
	struct node_por *next;
};

struct node_pro *list_father;//the node have already visited
struct node_pro *list_seq;//the list will be processed
struct node_pro *list_head_record;//the head of chuan


struct node_jguize{
	int len;
	int cnt;
	struct node_jguize *next;
};
struct node_jguize *list_jguize;


/******************************add-by-dt**************************/
////////////////////////////////////////////////////////////////////////////////

static void usage(char *name)
{
   fprintf(stderr,
           "Usage: %s [options] [<path>]\n"
           "   -a             dump all tables\n"
           "   -d xyz         dump selected tables in order given\n"
           "   -h             print usage\n"
	   "Tables:\n"
           "   T              tnum2tok\n"
           "   N              tok2tnum\n"
           "   a              arg2p\n"
           "   e              env2p\n"
           "   i              i2p\n"
           "   p              p2i\n"
           "   n              name\n"
           "   >              child\n"
           "   <              parent\n"
           "   d              provdb\n"
           "   m              rebuild\n"
           "   z                chakanguize\n"
           "   g               shengchengguize\n"
           "   j                jiance\n"
           "If no path is specified \".\" is used.\n"
           "If no tables are specified, all are dumped.\n",
           name );
}

/**
 * The main method for the wdb dump application.
 */
int
main(int argc, char *argv[])
{
   const  char          *path = ".";
   const  char          *dumps = "TNaeipn><dmg";
   int                   ch, i;
   struct timeval start,end;
   float timeopen;

   //
   // Parse options
   //
   while ( -1 != (ch = getopt(argc, argv, "ad:h")) ) {
      switch ( ch ) {
         case 'a':
	    dumps = "TNaeipn><dz";
	    break;
         case 'd':
	    dumps = optarg;
	    break;
         case 'h':
         default:
            usage(argv[0]);
            return 1;
      }
   }

   // extra params -> print usage
   if ( optind == argc - 1 ) {
      path = argv[optind];
   }
   else {
      usage(argv[0]);
      return 1;
   }

   //
   // Initialize all 'databases' (database & log)
   //
   gettimeofday(&start,NULL);
   if (wdb_startup(path, WDB_O_RDWR| WDB_O_CREAT) < 0) {
      fprintf(stderr, "wdb_dump: %s: %s\n", path, strerror(errno));
      exit(1);
   }
   gettimeofday(&end,NULL);
   timeopen = 1000000*(end.tv_sec-start.tv_sec)+end.tv_usec-start.tv_usec;
   //printf("======Open DB Time: %f microseconds , %f s ======\n",timeopen,timeopen/1000000);
   //
   // Dump everything
   //
   for (i=0; dumps[i]; i++) {
      dump_something(dumps[i]);
   }

   //
   // Shut everything down (database & log)
   //
   wdb_shutdown();

   exit(0);
}

static void printstring(const char *s, size_t len) {
   size_t i;
   putchar('"');
   for (i=0; i<len; i++) {
      unsigned char c = s[i];
      switch (c) {
       case '"':
       case '\\':
	putchar('\\');
	putchar(c);
	break;
       case '\a': putchar('\\'); putchar('a'); break;
       case '\b': putchar('\\'); putchar('b'); break;
       case '\t': putchar('\\'); putchar('t'); break;
       case '\n': putchar('\\'); putchar('n'); break;
       case '\v': putchar('\\'); putchar('v'); break;
       case '\f': putchar('\\'); putchar('f'); break;
       case '\r': putchar('\\'); putchar('r'); break;
       default:
	if (isascii(c) && (isprint(c) || isspace(c))) {
	   putchar(c);
	}
	else {
	   printf("\\%03o", c);
	}
	break;
      }
   }
   putchar('"');
}

static void printtoken(tnum_t tok) {
   DBT  dbt_tnum;
   DBT  dbt_token;
   int  ret;

   memset(&dbt_tnum, 0, sizeof(dbt_tnum));
   memset(&dbt_token, 0, sizeof(dbt_token));

   dbt_tnum.data = &tok;
   dbt_tnum.size = sizeof(tok);

   printf(" [%u]", tok);

   ret = g_tnum2tokdb->db->get(g_tnum2tokdb->db, NULL,
                               &dbt_tnum, &dbt_token,
                               0);
   wdb_check_err(ret);

   printstring(dbt_token.data, dbt_token.size);
}

/* For now, we use our own implementation of strnlen, since it's not clear
   what ifdefs we need here to make sure it is not already declared in libc */
static size_t wdb_strnlen(const char *s, size_t maxlen) {
   const char *t = memchr(s, 0, maxlen);
   return t ? (size_t)(t-s) : maxlen;
}

static void dump_something(int ch) {
   switch (ch) {
    case 'T': dump_tnum2tok(); break;
    case 'N': dump_tok2tnum(); break;
    case 'a': dump_arg2p(); break;
    case 'e': dump_env2p(); break;
    case 'i': dump_i2p(); break;
    case 'p': dump_p2i(); break;
    case 'n': dump_name(); break;
    case '>': dump_child(); break;
    case '<': dump_parent(); break;
    case 'd': dump_provdb(); break;
    case 'm': rebuild(); break;
    case 'g': guize();break;
    case 'z': dump_guize();break;
    case 'j': jiance_dt();break;
    case 'k':dump_ky2();break;
    default:
     fprintf(stderr, "Unknown dump target %c\n", ch);
     exit(1);
   }
}

void dump_provdb(void)
{
   DBC                       *pdbc = NULL;
   DBT                        dbt_key;
   DBT                        dbt_val;
   provdb_key                 key;
   provdb_val                *val;
   struct pnode_version      *ref;
   enum prov_attrname_packed  id = PROV_ATTR_INVALID;
   const char                *attrname = NULL;
   char                      *data;
   size_t                     i, datalen;
   int                        serrno;
   int                        ret;

   printf( "----------------------------------------\n"
           "| Dumping provdb                       |\n"
           "|                                      |\n"
           "----------------------------------------\n" );

   memset(&dbt_key, 0, sizeof(dbt_key));
   memset(&dbt_val, 0, sizeof(dbt_val));

   //
   // Process the entries
   //

   // Open a cursor
   ret = g_provdb->db->cursor(g_provdb->db, NULL, &pdbc, 0);
   wdb_check_err(ret);

   // Iterate over the main database
   while ( 0 == (ret = pdbc->c_get(pdbc, &dbt_key, &dbt_val, DB_NEXT)) ) {
      id = PROV_ATTR_INVALID;
      val = malloc(dbt_val.size);
      if ( val == NULL ) {
	serrno = errno;
	fprintf( stderr, "Malloc of provdb record failed %s\n",
		 strerror(serrno) );
	errno = serrno;
	// XXX clean up such as closing the cursor
	return;
      }

      memcpy(&key, dbt_key.data, sizeof(key));
      memcpy(val, dbt_val.data, dbt_val.size);

      assert( sizeof(key) == dbt_key.size );
#if DEBUG
      if ( sizeof(*val) >= dbt_val.size ) {
          fprintf( stderr,
                   "sizeof(*val) %lu dbt_val.size %d sizeof(provdb_val) %lu\n"
                   "key: %lu.%u\n"
                   "val: flags %02X valtype %02X code/len %02X/%d vallen %u\n",
                   sizeof(*val), dbt_val.size, sizeof(provdb_val),
                   key.pnum, key.version,
                   val->pdb_flags, val->pdb_valuetype,
                   val->pdb_attrcode, val->pdb_attrlen,
                   val->pdb_valuelen );
      }
#endif /* DEBUG */
      assert( sizeof(*val) <= dbt_val.size );

      attrname = PROVDB_VAL_ATTR(val);
      data = PROVDB_VAL_VALUE(val);
      datalen = PROVDB_VAL_VALUELEN(val);

      print_attrflags(val->pdb_flags);

      printf("%llu.%u %s %u ",
	     (unsigned long long)key.pnum, key.version,
             attrname, (unsigned)datalen);

      switch (PROVDB_VAL_VALUETYPE(val)) {

       case PROV_TYPE_NIL:
	printf("---\n");
	break;

       case PROV_TYPE_STRING:
	if (PROVDB_VAL_ISTOKENIZED(val)) {
	   tnum_t tok = *(tnum_t *) data;

	   assert(datalen == sizeof(tnum_t));
	   tok = *(tnum_t *) data;
	   printtoken(tok);
	   printf("\n");
	}
	else {
	   printstring(data, datalen);
	   printf("\n");
	}
	break;

       case PROV_TYPE_MULTISTRING:
	if (PROVDB_VAL_ISTOKENIZED(val)) {
	   int ntoks, i;
	   tnum_t *toks;

	   assert(datalen % sizeof(tnum_t) == 0);
	   ntoks = datalen / sizeof(tnum_t);
	   toks = malloc(datalen);
	   memcpy(toks, data, datalen);

	   for (i=0; i<ntoks; i++) {
	      printtoken(toks[i]);
	   }
	   free(toks), toks = NULL;
	}
	else {
	   size_t x, y;
	   for (x = 0; x < datalen; x += y+1) {
	      if (x > 0) printf(" ");
	      y = wdb_strnlen(data+x, datalen-x);
	      printstring(data+x, y);
	   }
	}
	printf("\n");
	break;

       case PROV_TYPE_INT:
	assert(datalen == sizeof(int32_t));
	printf( "INT %d\n", *(int32_t *)data );
	break;

       case PROV_TYPE_REAL:
	assert(datalen == sizeof(double));
	printf( "REAL %g\n", *(double *)data );
	break;

       case PROV_TYPE_TIMESTAMP:
	assert(datalen == sizeof(struct prov_timestamp));
	printf( "TIME %u.%09u\n",
		((struct prov_timestamp *)data)->pt_sec,
		((struct prov_timestamp *)data)->pt_nsec);
	break;

       case PROV_TYPE_INODE:
	assert(datalen == sizeof(uint32_t));
	printf( "INODE %u\n", *(uint32_t *)data );
	break;

       case PROV_TYPE_PNODE:
	assert(datalen == sizeof(pnode_t));
	printf( "--> %llu\n", (unsigned long long)*(pnode_t *)data);
	break;

       case PROV_TYPE_PNODEVERSION:
	assert(datalen == sizeof(struct pnode_version));
	ref = (struct pnode_version *)data;
	printf( "--> %llu.%u\n",
		(unsigned long long)ref->pnum, ref->version );
	break;

       case PROV_TYPE_OBJECT:
       case PROV_TYPE_OBJECTVERSION:
       default:
	printf("[illegal value type %u]", PROVDB_VAL_VALUETYPE(val));
	for (i=0; i<datalen; i++) {
	   printf(" %02x", (unsigned char) data[i]);
	}
	printf("\n");
      }

      free(val), val = NULL;
   }

   // XXX shouldn't this, like, close the cursor?

   if ( DB_NOTFOUND != ret ) {
      wdb_check_err(ret);
   }
}

static void print_attrflags(uint8_t attrflags) {
   printf("[%c%c%c] ",
	  ((attrflags & PROVDB_ANCESTRY)        ? 'A' : ' '),
	  ((attrflags & PROVDB_PACKED)          ? 'P' : ' '),
	  ((attrflags & PROVDB_TOKENIZED)       ? 'T' : ' '));
}

void dump_tnum2tok(void)
{
   DBC                  *tnum2tokc = NULL;
   DBT                   dbt_tnum;
   DBT                   dbt_token;
   db_recno_t            tnum;
   char                 *token;
   int                   ret;

   printf( "----------------------------------------\n"
           "| Dumping tnum2token                   |\n"
           "| tnum -> token                        |\n"
           "----------------------------------------\n" );

   memset(&dbt_tnum, 0, sizeof(dbt_tnum));
   memset(&dbt_token, 0, sizeof(dbt_token));

   //
   // Process the entries
   //

   // Open a cursor
   ret = g_tnum2tokdb->db->cursor(g_tnum2tokdb->db, NULL, &tnum2tokc, 0);
   wdb_check_err(ret);

   // Iterate over the main database
   while ( 0 == (ret = tnum2tokc->c_get(tnum2tokc, &dbt_tnum, &dbt_token, DB_NEXT)) ) {
      assert( sizeof(tnum) == dbt_tnum.size );

      token = malloc(dbt_token.size + 1);
      memset(token, 0, dbt_token.size + 1);
      memcpy(&tnum, dbt_tnum.data, dbt_tnum.size);
      memcpy(token, dbt_token.data, dbt_token.size);

      printf( "%u %s\n", tnum, token );
      free(token), token = NULL;
   }

   if ( DB_NOTFOUND != ret ) {
      wdb_check_err(ret);
   }
}

void dump_tok2tnum(void)
{
   DBC                  *tok2tnumc = NULL;
   DBT                   dbt_tnum;
   DBT                   dbt_token;
   db_recno_t            tnum;
   char                 *token;
   int                   ret;

   printf( "----------------------------------------\n"
           "| Dumping token2tnum                   |\n"
           "| token -> tnum                        |\n"
           "----------------------------------------\n" );

   memset(&dbt_tnum, 0, sizeof(dbt_tnum));
   memset(&dbt_token, 0, sizeof(dbt_token));

   //
   // Process the entries
   //

   // Open a cursor
   ret = g_tok2tnumdb->db->cursor(g_tok2tnumdb->db, NULL, &tok2tnumc, 0);
   wdb_check_err(ret);

   // Iterate over the main database
   while ( 0 == (ret = tok2tnumc->c_get(tok2tnumc, &dbt_token, &dbt_tnum, DB_NEXT)) ) {
      assert( sizeof(tnum) == dbt_tnum.size );

      token = malloc(dbt_token.size + 1);
      memset(token, 0, dbt_token.size + 1);
      memcpy(&tnum, dbt_tnum.data, dbt_tnum.size);
      memcpy(token, dbt_token.data, dbt_token.size);

      printf( "%s -> %u\n", token, tnum );
      free(token), token = NULL;
   }

   if ( DB_NOTFOUND != ret ) {
      wdb_check_err(ret);
   }
}

void dump_arg2p(void)
{
   DBC                  *arg2pc = NULL;
   DBT                   dbt_tnum;
   DBT                   dbt_pnode;
   db_recno_t            tnum;
   pnode_t               pnum;
   int                   ret;

   printf( "----------------------------------------\n"
           "| Dumping arg tnum -> pnode            |\n"
           "| arg tnum -> pnode                    |\n"
           "----------------------------------------\n" );

   memset(&dbt_tnum, 0, sizeof(dbt_tnum));
   memset(&dbt_pnode, 0, sizeof(dbt_pnode));

   //
   // Process the entries
   //

   // Open a cursor
   ret = g_arg2pdb->db->cursor(g_arg2pdb->db, NULL,
                               &arg2pc, 0);
   wdb_check_err(ret);

   // Iterate over the main database
   while ( 0 == (ret = arg2pc->c_get(arg2pc, &dbt_tnum, &dbt_pnode, DB_NEXT)) ) {
      assert( sizeof(tnum) == dbt_tnum.size );
      assert( sizeof(pnum) == dbt_pnode.size );

      memcpy(&tnum, dbt_tnum.data, dbt_tnum.size);
      memcpy(&pnum , dbt_pnode.data, dbt_pnode.size);

      printf( "%u -> %llu\n", (unsigned)tnum, (unsigned long long)pnum );
   }

   if ( DB_NOTFOUND != ret ) {
      wdb_check_err(ret);
   }
}

void dump_env2p(void)
{
   DBC                  *env2pc = NULL;
   DBT                   dbt_tnum;
   DBT                   dbt_pnode;
   db_recno_t            tnum;
   pnode_t               pnum;
   int                   ret;

   printf( "----------------------------------------\n"
           "| Dumping env tnum -> pnode            |\n"
           "| env tnum -> pnode                    |\n"
           "----------------------------------------\n" );

   memset(&dbt_tnum, 0, sizeof(dbt_tnum));
   memset(&dbt_pnode, 0, sizeof(dbt_pnode));

   //
   // Process the entries
   //

   // Open a cursor
   ret = g_env2pdb->db->cursor(g_env2pdb->db, NULL,
                               &env2pc, 0);
   wdb_check_err(ret);

   // Iterate over the main database
   while ( 0 == (ret = env2pc->c_get(env2pc, &dbt_tnum, &dbt_pnode, DB_NEXT)) ) {
      assert( sizeof(tnum) == dbt_tnum.size );
      assert( sizeof(pnum) == dbt_pnode.size );

      memcpy(&tnum, dbt_tnum.data, dbt_tnum.size);
      memcpy(&pnum , dbt_pnode.data, dbt_pnode.size);

      printf( "%u -> %llu\n", (unsigned)tnum, (unsigned long long)pnum );
   }

   if ( DB_NOTFOUND != ret ) {
      wdb_check_err(ret);
   }
}

void dump_name(void)
{
   DBC                  *namec = NULL;
   DBT                   dbt_name;
   DBT                   dbt_pnode;
   char                 *name;
   pnode_t               pnum;
   int                   ret;

   printf( "----------------------------------------\n"
           "| Dumping name                         |\n"
           "| name -> pnode                        |\n"
           "----------------------------------------\n" );

   memset(&dbt_name , 0, sizeof(dbt_name));
   memset(&dbt_pnode, 0, sizeof(dbt_pnode));

   //
   // Process the entries
   //

   // Open a cursor
   ret = g_namedb->db->cursor(g_namedb->db, NULL, &namec, 0);
   wdb_check_err(ret);

   // Iterate over the main database
   while ( 0 == (ret = namec->c_get(namec, &dbt_name, &dbt_pnode, DB_NEXT)) ) {
      assert( sizeof(pnum) == dbt_pnode.size );

      name = malloc(dbt_name.size + 1);
      memcpy(name, dbt_name.data, dbt_name.size);
      name[dbt_name.size] = 0;
      memcpy(&pnum, dbt_pnode.data, dbt_pnode.size);

      printf( "%s -> %llu\n", name, (unsigned long long)pnum );

      free(name), name = NULL;
   }

   if ( DB_NOTFOUND != ret ) {
      wdb_check_err(ret);
   }
}

void dump_i2p(void)
{
   DBC                  *i2pc = NULL;
   DBT                   dbt_inode;
   DBT                   dbt_pv;
   uint32_t              inode;
   struct pnode_version  pv;
   int                   ret;

   printf( "----------------------------------------\n"
           "| Dumping i2p                          |\n"
           "| ino -> pnode.version                 |\n"
           "----------------------------------------\n" );

   memset(&dbt_inode, 0, sizeof(dbt_inode));
   memset(&dbt_pv   , 0, sizeof(dbt_pv));

   //
   // Process the entries
   //

   // Open a cursor
   ret = g_i2pdb->db->cursor(g_i2pdb->db, NULL, &i2pc, 0);
   wdb_check_err(ret);

   // Iterate over the main database
   while ( 0 == (ret = i2pc->c_get(i2pc, &dbt_inode, &dbt_pv, DB_NEXT)) ) {
      assert( sizeof(inode) == dbt_inode.size );
      assert( sizeof(pv)    == dbt_pv.size );

      memcpy(&inode, dbt_inode.data, dbt_inode.size);
      memcpy(&pv , dbt_pv.data   , dbt_pv.size);

      printf( "%u -> %" PRIu64 ".%"  PRIu32 "\n",
              inode, pv.pnum, pv.version );
   }

   if ( DB_NOTFOUND != ret ) {
      wdb_check_err(ret);
   }
}

void dump_p2i(void)
{
   DBC                  *p2ic = NULL;
   DBT                   dbt_inode;
   DBT                   dbt_pnode;
   uint32_t              inode;
   pnode_t               pnum;
   int                   ret;

   printf( "----------------------------------------\n"
           "| Dumping p2i                          |\n"
           "| pnode -> in                          |\n"
           "----------------------------------------\n" );

   memset(&dbt_inode, 0, sizeof(dbt_inode));
   memset(&dbt_pnode, 0, sizeof(dbt_pnode));

   //
   // Process the entries
   //

   // Open a cursor
   ret = g_p2idb->db->cursor(g_p2idb->db, NULL, &p2ic, 0);
   wdb_check_err(ret);

   // Iterate over the main database
   while ( 0 == (ret = p2ic->c_get(p2ic, &dbt_pnode, &dbt_inode, DB_NEXT)) ) {
      assert( sizeof(inode) == dbt_inode.size );
      assert( sizeof(pnum)  == dbt_pnode.size );

      memcpy(&inode, dbt_inode.data, dbt_inode.size);
      memcpy(&pnum , dbt_pnode.data, dbt_pnode.size);

      printf( "%llu -> %u\n", (unsigned long long)pnum, inode );
   }

   if ( DB_NOTFOUND != ret ) {
      wdb_check_err(ret);
   }
}

void dump_child(void)
{
   DBC                  *childc = NULL;
   DBT                   dbt_parent;
   DBT                   dbt_child;
   provdb_key            parent;
   provdb_key            child;
   int                   ret;

   printf( "----------------------------------------\n"
           "| Dumping child                        |\n"
           "| parent -> child                      |\n"
           "----------------------------------------\n" );

   memset(&dbt_parent, 0, sizeof(dbt_parent));
   memset(&dbt_child , 0, sizeof(dbt_child));

   //
   // Process the entries
   //

   // Open a cursor
   ret = g_childdb->db->cursor(g_childdb->db, NULL, &childc, 0);
   wdb_check_err(ret);

   // Iterate over the main database
   while ( 0 == (ret = childc->c_get(childc, &dbt_parent, &dbt_child, DB_NEXT)) ) {
      assert( sizeof(parent) == dbt_parent.size );
      assert( sizeof(child)  == dbt_child.size );

      memcpy(&parent, dbt_parent.data, dbt_parent.size);
      memcpy(&child , dbt_child.data , dbt_child.size);

      printf( "%llu.%u -> %llu.%u\n",
              (unsigned long long)parent.pnum, parent.version,
              (unsigned long long)child.pnum , child.version );
   }

   if ( DB_NOTFOUND != ret ) {
      wdb_check_err(ret);
   }
}

void dump_parent(void)
{
   DBC                  *parentc = NULL;
   DBT                   dbt_child;
   DBT                   dbt_parent;
   provdb_key            child;
   provdb_key            parent;
   int                   ret;

   printf( "----------------------------------------\n"
           "| Dumping parent                        |\n"
           "| child -> parent                      |\n"
           "----------------------------------------\n" );

   memset(&dbt_child, 0, sizeof(dbt_child));
   memset(&dbt_parent , 0, sizeof(dbt_parent));

   //
   // Process the entries
   //

   // Open a cursor
   ret = g_parentdb->db->cursor(g_parentdb->db, NULL, &parentc, 0);
   wdb_check_err(ret);

   // Iterate over the main database
   while ( 0 == (ret = parentc->c_get(parentc, &dbt_child, &dbt_parent, DB_NEXT)) ) {
      assert( sizeof(child) == dbt_child.size );
      assert( sizeof(parent)  == dbt_parent.size );

      memcpy(&child , dbt_child.data , dbt_child.size );
      memcpy(&parent, dbt_parent.data, dbt_parent.size);

      printf( "%llu.%u -> %llu.%u\n",
              (unsigned long long)child.pnum, child.version,
              (unsigned long long)parent.pnum , parent.version );
   }

   if ( DB_NOTFOUND != ret ) {
      wdb_check_err(ret);
   }
}

void dump_guize(void)
{
   DBC                  *namec = NULL;
   DBT                   dbt_name;
   DBT                   dbt_pnode;
   char                 *name;
   char                  *pnum;
   int                   ret;
   int                    num;

   printf( "----------------------------------------\n"
           "| Dumping guize                         |\n"
           "| name -> name                        |\n"
           "----------------------------------------\n" );

   memset(&dbt_name , 0, sizeof(dbt_name));
   memset(&dbt_pnode, 0, sizeof(dbt_pnode));

   //
   // Process the entries
   //

   // Open a cursor
   ret = g_guizedb->db->cursor(g_guizedb->db, NULL, &namec, 0);
   wdb_check_err(ret);
   num=0;

   // Iterate over the main database
   while ( 0 == (ret = namec->c_get(namec, &dbt_name, &dbt_pnode, DB_NEXT)) ) {

      name = malloc(dbt_name.size + 1);
      memcpy(name, dbt_name.data, dbt_name.size);
      name[dbt_name.size] = 0;
	  
      pnum = malloc(dbt_pnode.size + 1);
      memcpy(pnum, dbt_pnode.data, dbt_pnode.size);
      pnum[dbt_pnode.size] = 0;
      printf( "%s -> %s\n", name, pnum );
      num++;

      free(name), name = NULL;
      free(pnum), pnum = NULL;

   }

   printf("num:%d\n",num);
   if ( DB_NOTFOUND != ret ) {
      wdb_check_err(ret);
   }
}
void dump_ky2(void)
{
   DBC                  *p2ic = NULL;
   DBT                   dbt_inode;
   DBT                   dbt_pnode;
   pnode_t              inode;
   pnode_t               pnum;
   int                   ret;

   printf( "----------------------------------------\n"
           "| Dumping ky2                          |\n"
           "| c -> p                          |\n"
           "----------------------------------------\n" );

   memset(&dbt_inode, 0, sizeof(dbt_inode));
   memset(&dbt_pnode, 0, sizeof(dbt_pnode));

   //
   // Process the entries
   //

   // Open a cursor
   ret = g_ky2db->db->cursor(g_ky2db->db, NULL, &p2ic, 0);
   wdb_check_err(ret);

   // Iterate over the main database
   while ( 0 == (ret = p2ic->c_get(p2ic, &dbt_pnode, &dbt_inode, DB_NEXT)) ) {
      assert( sizeof(inode) == dbt_inode.size );
      assert( sizeof(pnum)  == dbt_pnode.size );

      memcpy(&inode, dbt_inode.data, dbt_inode.size);
      memcpy(&pnum , dbt_pnode.data, dbt_pnode.size);

      printf( "%llu <- %llu\n", pnum, inode );
   }

   if ( DB_NOTFOUND != ret ) {
      wdb_check_err(ret);
   }
}

//ret=1,not find
static int find(char * pname ,char * cname)
{
   DBC                       *guizec =NULL;
   DBT                        dbt_guize_key;
   DBT                        dbt_guize_val;
   char                        *c;
   int                           ret,r=1;

   memset(&dbt_guize_key, 0, sizeof(dbt_guize_key));
   memset(&dbt_guize_val, 0, sizeof(dbt_guize_val));

   ret = g_guizedb->db->cursor(g_guizedb->db, NULL, &guizec, 0);
   wdb_check_err(ret);

   dbt_guize_key.data = pname;
   dbt_guize_key.size = strlen(pname);
   ret = guizec->c_get(guizec, &dbt_guize_key, &dbt_guize_val, DB_SET);
   while ( ret != DB_NOTFOUND ) {
   	c = malloc(dbt_guize_val.size+1);
	memcpy(c, dbt_guize_val.data, dbt_guize_val.size);
	c[dbt_guize_val.size] = 0; 
	if( strcmp(c, cname) ==0 ) {r=0;free(c);c=NULL;break;}
	ret = guizec->c_get(guizec, &dbt_guize_key, &dbt_guize_val, DB_NEXT_DUP);
	free(c),c=NULL;
   }
   return r;

}

static int insert_detect(pnode_t pname , pnode_t cname)
{
printf("Insert: %llu->%llu\n",pname,cname);
   DBC                  *ky1c = NULL;
   DBC                  *ky2c = NULL;
   DBC                  *ky1c1 = NULL;
   DBC                  *ky2c1 = NULL;
   DBT                   dbt_child;
   DBT                   dbt_parent;
   DBT                   dbt_child1;
   DBT                   dbt_parent1;
   pnode_t               c,c1;
   int                     len=0,len1=0;
   int                     ret,ret1;

   memset(&dbt_child, 0, sizeof(dbt_child));
   memset(&dbt_parent , 0, sizeof(dbt_parent));
   memset(&dbt_child1, 0, sizeof(dbt_child1));
   memset(&dbt_parent1 , 0, sizeof(dbt_parent1));

   dbt_parent.data = &pname;
   dbt_parent.size = sizeof(pname);

   dbt_child.data = &cname;
   dbt_child.size = sizeof(cname);

   ret = g_ky1db->db->put(g_ky1db->db, NULL,
                           &dbt_parent, &dbt_child,
                           g_ky1db->put_flags);
   if ( ret == DB_KEYEXIST ) {
#ifdef DEBUG
       fprintf( stderr,
	        "name index: key %*s for pnode %llu already recorded\n",
                length, val->pdb_data, key->pnum );
#endif /* DEBUG */
   } else {
       wdb_check_err(ret);
   }

   ret = g_ky2db->db->put(g_ky2db->db, NULL,
                           &dbt_child, &dbt_parent,
                           g_ky2db->put_flags);

   if ( ret == DB_KEYEXIST ) {
#ifdef DEBUG
       fprintf( stderr,
	        "name index: key %*s for pnode %llu already recorded\n",
                length, val->pdb_data, key->pnum );
#endif /* DEBUG */
   } else {
       wdb_check_err(ret);
   }

   // Open a cursor
   ret = g_ky1db->db->cursor(g_ky1db->db, NULL, &ky1c, 0);
   wdb_check_err(ret);
   ret = g_ky2db->db->cursor(g_ky2db->db, NULL, &ky2c, 0);
   wdb_check_err(ret);
   ret = g_ky1db->db->cursor(g_ky1db->db, NULL, &ky1c1, 0);
   wdb_check_err(ret);
   ret = g_ky2db->db->cursor(g_ky2db->db, NULL, &ky2c1, 0);
   wdb_check_err(ret);

   dbt_parent.data = &cname;
   dbt_parent.size = sizeof(cname);
   ret = ky1c->c_get(ky1c, &dbt_parent, &dbt_child, DB_SET);
   while ( ret != DB_NOTFOUND ) {
   	if(len<1) len=1;
	memcpy(&c, dbt_child.data, dbt_child.size);
	dbt_parent1.data = &c;
	dbt_parent1.size = sizeof(c);
	ret1 = ky1c1->c_get(ky1c1, &dbt_parent1, &dbt_child1, DB_SET);
	while ( ret1 != DB_NOTFOUND ) {
		len=2;
		memcpy(&c1, dbt_child1.data, dbt_child1.size);
		printf("-> %llu -> %llu\n",c,c1);
		ret1 = ky1c1->c_get(ky1c1, &dbt_parent1, &dbt_child1, DB_NEXT_DUP);
	}
	ret = ky1c->c_get(ky1c, &dbt_parent, &dbt_child, DB_NEXT_DUP);
   }

   dbt_child.data = &pname;
   dbt_child.size = sizeof(pname);
   ret = ky2c->c_get(ky2c, &dbt_child, &dbt_parent, DB_SET);
   while ( ret != DB_NOTFOUND ) {
   	if(len1<1) len1=1;
	memcpy(&c, dbt_parent.data, dbt_parent.size);
	dbt_child1.data = &c;
	dbt_child1.size = sizeof(c);
	ret1 = ky2c1->c_get(ky2c1, &dbt_child1, &dbt_parent1, DB_SET);
	while ( ret1 != DB_NOTFOUND ) {
		len1=2;
		memcpy(&c1, dbt_parent1.data, dbt_parent1.size);
		printf("%llu -> %llu ->\n",c1,c);
		ret1 = ky2c1->c_get(ky2c1, &dbt_child1, &dbt_parent1, DB_NEXT_DUP);
	}
	ret = ky2c->c_get(ky2c, &dbt_child, &dbt_parent, DB_NEXT_DUP);
   }

printf("->%d, <-%d\n",len,len1);
   
   return 0;
}

void jiance_dt()
{
	//here to calc the run time,include two parts:gernerate guize & jiance time.
	start=clock(); 
	j=1;
	
	list_jguize_init();
	guize();

//after this function called,the list_jguize is filled with   len,data,call the calc funtion to computer the possibility;
	
	cal_similiar_possibility();
	
	duration=clock()-start;
	printf("系统运行时间: %dms\n",duration*1000/CLOCKS_PER_SEC);

}
double jieceng(int n)
{
	double sum=1;
	for(;n>0;--n)
		sum*=(double)(n);

	return sum;
}
double power(double base,int exp)
{
	int i=0;
	double sum=1;
	for(i=0;i<exp;++i)
		sum*=base;

	return sum;
}
double possibility(int len,int cnt)
{
	 double delter=(double)(len)/6.0;
        double miu=(double)(len)/2.0;
        int index=0;
        int sign=0;
        double value=0.0;
        double sum=0.0;

        value=((double)(cnt)-miu)/delter;
        if(value<0)
        {
                sign=1;
                value=-value;
        }

        index=value*100;
        if(sign==0)
                sum=ztfb[index];
        else
                sum=1-ztfb[index];

        return sum;
}
int cal_similiar_possibility()
{
	 struct node_jguize *p=list_jguize->next;
        double pos1=0.0;
        double pos2=0.0;

        double sum1=0.0;
        double sum2=0.0;

        double tmp1=0.0;
        double tmp2=0.0;

        double weight=0.0;
        int length=0;

        while(p)
        {
                length+=p->len;
                p=p->next;
        }

        printf("len     cnt     weight     pos1   pos2\n");

        p=list_jguize->next;
        while(p)
        {
                pos1=possibility(p->len,p->cnt);
                pos2=(double)(p->cnt)/(double)(p->len);

                weight=(double)(p->len)/(double)(length);
                tmp1=pos1*weight;
                tmp2=pos2*weight;

                sum1+=tmp1;
                sum2+=tmp2;
                printf("%d       %d       %lf       %lf        %lf\n",p->len,p->cnt,weight,pos1,pos2);
                p=p->next;
        }

        printf("改进溯源相似度为：%lf;\nLCS相似度为:%lf\n",sum1,sum2);
	
	printf("阈值设定为0.5,检测结果及检测时间如下：\n");
	if(sum1>0.5)
		printf("改进溯源检测到入侵行为\n");
	else 
		printf("改进溯源未检测到入侵行为\n");

	if(sum2>0.5)
		printf("系统调用检测到入侵行为\n");
	else
		printf("系统调用未检测到入侵行为\n");
        return 0;

	
}
void list_jguize_init()
{
	list_jguize=(struct node_jguize *)malloc(sizeof(struct node_jguize));
	list_jguize->next=NULL;
	list_jguize->len=0;//the head of this list node,the len attr is the length of the list;
	list_jguize->cnt=0;
	FILE *file=NULL;
	char buf[1024];
	int num=0;
	int index;
	file=fopen("/dt/guize.txt","r");
	if(file!=NULL)
	{
		while(fgets(buf,1024,file))
		{
			buf[strlen(buf)-1]=0;
			num=atoi(buf);
			list_jguize_add(num);
			for(index=0;index<num;++index)
			{
				fgets(buf,1024,file);
				buf[strlen(buf)-1]=0;
			}
		}
		
	}
	fclose(file);	
	
}
void list_jguize_add(int num)
{
	struct node_jguize *p=NULL;
	struct node_jguize *q=NULL;
	p=(struct node_jguize *)malloc(sizeof(struct node_jguize));
	p->next=NULL;
	p->len=num;
	p->cnt=0;

	list_jguize->len++;

	q=list_jguize;
	
	while(q->next)
		q=q->next;

	q->next=p;
}
void jiance_list_seq(struct node_pro * head)
{
	struct node_jguize *p_jguize;
	struct node_pro *p_pro;
	p_pro=(struct node_pro *)malloc(sizeof(struct node_pro));
	p_pro->next=NULL;
	p_pro->name=NULL;

	p_jguize=list_jguize->next;
	
	
	FILE *file=NULL;
	char buf[1024];
	int num=0;
	int index=0;

	int max_cnt=0;
	file=fopen("/dt/guize.txt","r");

	
	if(file!=NULL)
	{
		while(fgets(buf,1024,file))
		{
			buf[strlen(buf)-1]=0;// delete the \n charcter
			num=atoi(buf);
			
			
			for(index=0;index<num;++index)
			{
				fgets(buf,1024,file);
				buf[strlen(buf)-1]=0;
				add_list(p_pro, buf);// insert node_pro information to the list
			}
			
			max_cnt=find_max_seq_num(head, p_pro);
			clear_list(p_pro);
			
			if((p_jguize->cnt < max_cnt)  && (p_jguize->len > max_cnt))
				p_jguize->cnt=max_cnt;
			if(p_jguize->next)
				p_jguize=p_jguize->next;
			
		}
		
	}

	free(p_pro);
	fclose(file);
	
}
int find_max_seq_num(struct node_pro *L1,struct node_pro *L2)
{
	struct node_pro *p1=NULL;
	struct node_pro *p2=NULL;
	int m=0;
	int n=0;
	int index1,index2;
	int max_len=0;
	p1=L1->next;
	while(p1)
	{
		m++;
		p1=p1->next;
	}

	p2=L2->next;
	while(p2)
	{
		n++;
		p2=p2->next;
	}

	if(m>100)
		m=100;
	if(n>100)
		n=100;
	int arr[101][101]={{0},{0}};

	p1=L1->next;
	p2=L2->next;
	
	for(index1=1;index1<=m;++index1)
	{
		for(index2=1;index2<=n;++index2)
		{
			if(strcmp(p1->name,p2->name)==0)
				arr[index1][index2]=arr[index1-1][index2-1]+1;
			else
			{
				if(arr[index1][index2-1]>=arr[index1-1][index2])
					arr[index1][index2]=arr[index1][index2-1];
				else
					arr[index1][index2]=arr[index1-1][index2];
			}
			
			p2=p2->next;
			if(p2==NULL)
				p2=L2->next;
			
		}

		p1=p1->next;
		if(p1==NULL)
			p1=L1->next;

	}

//find the max num of the arr,this mean the max longth of the two list;


	for(index1=0;index1<=m;++index1)
		for(index2=0;index2<=n;++index2)
		{
			if(arr[index1][index2]>max_len)
				max_len=arr[index1][index2];
		}

	return max_len;	
}

static void jiance(void)
{
   DBC                  *jchildc = NULL;
   DBT                   dbt_jchild;
   DBT                   dbt_jparent;
   provdb_key            jchild;
   provdb_key            jparent;
   int                   ret;
   char *  pname;
   char *  cname;

   memset(&dbt_jchild, 0, sizeof(dbt_jchild));
   memset(&dbt_jparent , 0, sizeof(dbt_jparent));
   j=1;
   //
   // Process the entries
   //

   // Open a cursor
   ret = g_jchilddb->db->cursor(g_jchilddb->db, NULL, &jchildc, 0);
   wdb_check_err(ret);

   // Iterate over the main database
   while ( 0 == (ret = jchildc->c_get(jchildc, &dbt_jparent, &dbt_jchild, DB_NEXT)) ) {
      assert( sizeof(jchild) == dbt_jchild.size );
      assert( sizeof(jparent)  == dbt_jparent.size );

      memcpy(&jchild , dbt_jchild.data , dbt_jchild.size );
      memcpy(&jparent, dbt_jparent.data, dbt_jparent.size);

     pname = get_string(jparent.pnum,"NAME");
     if(!pname)
        {
        	pname = get_string(jparent.pnum,"TYPE");
		if(!pname) continue;
	 }
     cname = get_string(jchild.pnum, "NAME");
     if(!cname)
        {
        	cname = get_string(jchild.pnum,"TYPE");
		if(!cname) 
		{free(pname),pname=NULL;
	 	 continue;}
	 }

   int r;
   r= find(pname,cname);//pipeishifoucunzai
   if(r)//mei you zhao dao 
   r= insert_detect(jparent.pnum,jchild.pnum);//jianceshifouchangdu=3
   
	 free(cname),cname=NULL;
	 free(pname),pname=NULL;

   }

   if ( DB_NOTFOUND != ret ) {
      wdb_check_err(ret);
   }



}



static void guize(void)
{
   DBC                  *childc = NULL;
   DBT                   dbt_child;
   DBT                   dbt_parent;
   provdb_key            child;
   provdb_key            parent;
   int                   ret,ret1,ret2;//ret2 is add by dt
   char *  pname;
   char *  ptype;
   char *  cname;
   char *  ctype;
   struct waldo_db * tmp_childdb;
   struct waldo_db * tmp_guizedb;
   struct waldo_db * tmp_guizerdb;
   
   memset(&dbt_child, 0, sizeof(dbt_child));
   memset(&dbt_parent , 0, sizeof(dbt_parent));

   //
   // Process the entries
   //

   // Open a cursor
   //if j==1 process the g_jchilddb,else process the g_childdb; 
   if(j==1) 
  	tmp_childdb=g_jchilddb;
   else
   	tmp_childdb=g_childdb;
   
   ret = tmp_childdb->db->cursor(tmp_childdb->db, NULL, &childc, 0);
   wdb_check_err(ret);

   // Iterate over the main database
   while ( 0 == (ret = childc->c_get(childc, &dbt_parent, &dbt_child, DB_NEXT)) ) {
      assert( sizeof(child) == dbt_child.size );
      assert( sizeof(parent)  == dbt_parent.size );

      memcpy(&child , dbt_child.data , dbt_child.size );
      memcpy(&parent, dbt_parent.data, dbt_parent.size);

     pname = get_string(parent.pnum,"NAME");
     ptype   =get_string(parent.pnum,"TYPE"); 
     
     if(!pname)
        {
		pname=get_string(parent.pnum,"TYPE"); 
		if(!pname)
		{

			pname = get_string(parent.pnum,"INPUT");
			if(pname)
			{
				free(pname);
				pname=malloc(sizeof(char)*5);
				strcpy(pname,"PIPE");
			}
			else
			{
				if(pname){free(pname);pname=NULL;}
				continue;
			}
		}
	 }

	 
     cname = get_string(child.pnum, "NAME");
     if(!cname)
        {
		cname=get_string(child.pnum, "TYPE");
		if(!cname)
		{

			cname = get_string(child.pnum,"INPUT");
			if(cname)
			{
				free(cname);
				cname=malloc(sizeof(char)*5);
				strcpy(cname,"PIPE");
			}
			else
			{
				if(pname){free(pname);pname=NULL;}
				if(cname){free(cname);cname=NULL;}
				continue;
			}
		}
	 }

//	printf("pname-->cname:%s--->%s\n",pname,cname);
   if(strcmp(pname,cname)==0)
   	continue;
   
   DBT                        dbt_guize_key;
   DBT                        dbt_guize_val;

   memset(&dbt_guize_key, 0, sizeof(dbt_guize_key));
   memset(&dbt_guize_val, 0, sizeof(dbt_guize_val));

   dbt_guize_key.data = pname;
   dbt_guize_key.size = strlen(pname);

   dbt_guize_val.data = cname;
   dbt_guize_val.size = strlen(cname);

	if(j==1)
		tmp_guizedb=g_jguizedb;
	else 
		tmp_guizedb=g_guizedb;
	
   ret1 = tmp_guizedb->db->put(tmp_guizedb->db, NULL,
                           &dbt_guize_key, &dbt_guize_val,
                           tmp_guizedb->put_flags);

/***************add-by-dt*********************/
//c->p infromation is store in this db

	if(j==1)
		tmp_guizerdb=g_jguizerdb;
	else
		tmp_guizerdb=g_guizerdb;
	
  ret2=tmp_guizerdb->db->put(tmp_guizerdb->db,NULL,&dbt_guize_val,&dbt_guize_key,tmp_guizerdb->put_flags);
/********************add-by-dt*******************/

	 free(cname),cname=NULL;
	 free(pname),pname=NULL;

   }

   if ( DB_NOTFOUND != ret ) {
      wdb_check_err(ret);
   }


/***********add-by-dt*********************/ //call function to procuer this prov inf to list,add store in the file.
//this function returned with the list_head_record store the head of a list.
guize_get_head_list();



guize_process();

}

/*******************add-by-dt******************/
// clear up the list_father list,and get the head list node.
int clear_list(struct node_pro * head)
{
	struct node_pro *p=head->next;
	struct node_pro *del=NULL;
	while(p)
		{
		del=p;
		p=p->next;
		
		free(del->name);
		free(del);
	}
	
	head->next=NULL;
	return 0;
}
int list_store(struct node_pro * head)
{
	struct node_pro *p;
	int num=0;
	p=head->next;
	while(p)
		{
		num++;
		p=p->next;
	}
	printf("%d\n",num);

	p=head->next;
	while(p)
		{
		printf("%s\n",p->name);
		p=p->next;
	}

	return 0;
}
int add_list(struct node_pro *head,char *name)
{
	struct node_pro *p=head;
	struct node_pro *q=(struct node_pro *)malloc(sizeof(struct node_pro));
	q->next=NULL;
	q->name=(char *)malloc(sizeof(char)*strlen(name)+1);
	memcpy(q->name,name,strlen(name));
	q->name[strlen(name)]=0;
	while(p->next)
		p=p->next;
	p->next=q;

	return 0;
}
//only delete the last node of the list
int delete_list(struct node_pro *head)
{
	struct node_pro *p=head;
	struct node_pro *q=NULL;

	while(p->next)
		{
		q=p;
		p=p->next;
	}
	if(q)
	{
		q->next=NULL;
		free(p->name);
		free(p);
	}

	return 0;
}
int guize_process()
{
	struct node_pro *p=NULL;
	struct node_pro *q=NULL;
	
	clear_list(list_father);

	//init list_seq;
	q=(struct node_pro *)malloc(sizeof(struct node_pro));
	q->next=NULL;
	q->name=NULL;
	
	list_seq=q;
	
	p=list_head_record->next;
	while(p)
	{
		
		get_seq(list_seq, p->name, 0);
		p=p->next;
	}


	// TODO:   free the list_head_record list here;
	return 0;
}
int found_list_father(struct node_pro * head,char * name)
{
	int flag=0;
	struct node_pro *p=head->next;
	while(p)
	{
			if(strcmp(p->name,name)==0)
			{
				flag=1;
				return flag;
			}
			p=p->next;
	}

	return flag;
}
//mei ge chuan de top jiedian cunfang zai list_head_record
int  get_head_record(char * cname)
{
   DBC                       *guizec =NULL;
   DBT                        dbt_guize_key;
   DBT                        dbt_guize_val;
   char                        *pname;
   int                           ret,r=0;
   struct waldo_db * tmp_guizerdb;
   
   struct node_pro *p;
   struct node_pro *q;

   struct node_pro *p1,*q1;
   memset(&dbt_guize_key, 0, sizeof(dbt_guize_key));
   memset(&dbt_guize_val, 0, sizeof(dbt_guize_val));


   if(!found_list_father(list_father,cname))// not visited ,add to the list_father list,else visited alyet return ;
   	{
	q1=(struct node_pro *)malloc(sizeof(struct node_pro));
	q1->name=(char *)malloc(sizeof(char)*strlen(cname)+1);
	memcpy(q1->name,cname,strlen(cname));
	q1->name[strlen(cname)]=0;
	//q1->name=cname;
	
	q1->next=NULL;
	p1=list_father;
	while(p1->next)
		p1=p1->next;
	
	p1->next=q1;
   }
   else
   	{return 0;}

   if(j==1)
		tmp_guizerdb=g_jguizerdb;
   else 
		tmp_guizerdb=g_guizerdb;
   
   ret = tmp_guizerdb->db->cursor(tmp_guizerdb->db, NULL, &guizec, 0);
   wdb_check_err(ret);

   dbt_guize_key.data = cname;
   dbt_guize_key.size = strlen(cname);
   ret = guizec->c_get(guizec, &dbt_guize_key, &dbt_guize_val, DB_SET);

   if(ret==DB_NOTFOUND){
	q=(struct node_pro *)malloc(sizeof(struct node_pro));
	q->name=(char *)malloc(sizeof(char)*strlen(cname)+1);
	memcpy(q->name,cname,strlen(cname));
	q->name[strlen(cname)]=0;
	//q1->name=cname;
	
	q->next=NULL;
	p=list_head_record;
	while(p->next)
		p=p->next;
	
	p->next=q;
	
	return 0;
   }
   while ( ret != DB_NOTFOUND ) {
   	pname = malloc(dbt_guize_val.size+1);
	memcpy(pname, dbt_guize_val.data, dbt_guize_val.size);
	pname[dbt_guize_val.size] = 0; 
	//if( strcmp(c, cname) ==0 ) {r=0;free(c);c=NULL;break;}
	get_head_record(pname);
	ret = guizec->c_get(guizec, &dbt_guize_key, &dbt_guize_val, DB_NEXT_DUP);
	
	free(pname),pname=NULL;
   }
   return 0;
}
int guize_get_head_list()
{
   DBC                  *namec = NULL;
   DBT                   dbt_cname;
   DBT                   dbt_pname;
   char                 *cname;
   char                  *pname;
   int                   ret;
   int                    num;
   int                   flag;
   struct waldo_db * tmp_guizedb;
   
   memset(&dbt_cname , 0, sizeof(dbt_cname));
   memset(&dbt_pname, 0, sizeof(dbt_pname));

   if(j==1)
		tmp_guizedb=g_jguizedb;
   else 
		tmp_guizedb=g_guizedb;
   
 ret = tmp_guizedb->db->cursor(tmp_guizedb->db, NULL, &namec, 0);
   wdb_check_err(ret);
   num=0;
   
	list_father=(struct node_pro *)malloc(sizeof(struct node_pro));
	list_father->next=NULL;
	list_seq=(struct node_pro *)malloc( sizeof(struct node_pro));
	list_seq->next=NULL;
	list_head_record=(struct node_pro *)malloc( sizeof(struct node_pro));
	list_head_record->next=NULL;
	
   // Iterate over the c->p  database
   while ( 0 == (ret = namec->c_get(namec, &dbt_cname, &dbt_pname, DB_NEXT)) ) {

      cname = malloc(dbt_cname.size + 1);
      memcpy(cname, dbt_cname.data, dbt_cname.size);
      cname[dbt_cname.size] = 0;


	  
      pname = malloc(dbt_pname.size + 1);
      memcpy(pname, dbt_pname.data, dbt_pname.size);
      pname[dbt_pname.size] = 0;
	
	get_head_record(cname);
		
    //  printf( "%s -> %s\n", name, pnum );
   //   num++;   
      free(cname), cname = NULL;
      free(pname), pname = NULL;

   }
	
   if ( DB_NOTFOUND != ret ) {
      wdb_check_err(ret);
   }

	
}
int get_seq(struct node_pro *head,char *pname,int flag)
{
   DBC                       *guizec =NULL;
   DBT                        dbt_guize_key;
   DBT                        dbt_guize_val;
   //char                        *pname;
   char                        *cname;
   int                           ret;

    struct waldo_db * tmp_guizedb;

    if(found_list_father(head, pname))//it may be a circle ,to deal this condition,if this value is ture,then print this list;
   	{
		if(j==1)
			jiance_list_seq(head);// compare the list with the guize ku;
		else
			list_store(head);
		return 0;
   }

   add_list(head,pname);
  
   memset(&dbt_guize_key, 0, sizeof(dbt_guize_key));
   memset(&dbt_guize_val, 0, sizeof(dbt_guize_val));

   if(j==1)
   	tmp_guizedb=g_jguizedb;
   else
   	tmp_guizedb=g_guizedb;
   ret = tmp_guizedb->db->cursor(tmp_guizedb->db, NULL, &guizec, 0);
   wdb_check_err(ret);

  
   dbt_guize_key.data = pname;
   dbt_guize_key.size = strlen(pname);
   ret = guizec->c_get(guizec, &dbt_guize_key, &dbt_guize_val, DB_SET);
   if(ret==DB_NOTFOUND)//this means this path is run into the leaf,should store this path in files; //add by dt
   {
	if(j==1)
		jiance_list_seq(head);// compare the list with the guize ku;
	else
		list_store(head);
		
	delete_list(head);
	return 0;
   }
   
   while ( ret != DB_NOTFOUND ) {
   	cname = malloc(dbt_guize_val.size+1);
	memcpy(cname, dbt_guize_val.data, dbt_guize_val.size);
	cname[dbt_guize_val.size] = 0; 
	
	get_seq(head, cname, 0);
	
	ret = guizec->c_get(guizec, &dbt_guize_key, &dbt_guize_val, DB_NEXT_DUP);
	
	free(cname),cname=NULL;
   }

   delete_list(head);
   return 0;		

}

/*********************add-by-dt**********************/
static void rebuild(void)
{
   char file_name[60] ;
   struct pnode_version pv,proc;
   int ret, state_num,j;
   struct timeval start,end;
   float t;

     printf("Input the rebuild file name :\n");
     scanf("%s",file_name+1);
     file_name[0]='/';
     pv.pnum =0;
     pv.version =0;
	 
/*-------------setp 1: <find>------------------*/
     gettimeofday(&start,NULL);
     ret = get_file_pv(file_name, &pv);
     if( !ret ) {
   	  printf("The file can't be rebuild.PASS didn't collected its provrence.\n");
	  return ;
     }
     else printf("The file's pnode.version is %llu.%u\n",pv.pnum,pv.version);
     proc = execute(pv);
     gettimeofday(&end,NULL);
     t = 1000000*(end.tv_sec-start.tv_sec)+end.tv_usec-start.tv_usec;
printf("======Search Time: %f microseconds , %f s ======\n",t,t/1000000);

/*-------------setp 2: <analysis>------------------*/
     gettimeofday(&start,NULL);
     state_num = analysis(proc);
     gettimeofday(&end,NULL);
     t = 1000000*(end.tv_sec-start.tv_sec)+end.tv_usec-start.tv_usec;
printf("\n======Analysis Time: %f microseconds , %f s ======\n",t,t/1000000);
printf("Number of stat %u\n",state_num);
int a=0,b=0,c=0;
for(j=0;j<state_num;j++){
printf("%u  ",state[j]);
if(state[j]==0) a++;
else if(state[j]==1) b++;
	else c++;
}
printf("\nRegular:%u, Delete:%u,Version:%u\n",a,b,c);

/*-------------setp 3: <execute>------------------*/
   gettimeofday(&start,NULL);
   proc_rebuild(proc.pnum);
   gettimeofday(&end,NULL);
   t = 1000000*(end.tv_sec-start.tv_sec)+end.tv_usec-start.tv_usec;
printf("======Execute Time: %f microseconds , %f s ======\n",t,t/1000000);

/*-------------setp 4: <recover state>------------------*/
   gettimeofday(&start,NULL);
   recover(state_num);
   gettimeofday(&end,NULL);
   t = 1000000*(end.tv_sec-start.tv_sec)+end.tv_usec-start.tv_usec;
printf("======State recover Time: %f microseconds , %f s ======\n",t,t/1000000);
}

/*By given the pnum & version, we first find the name in the provdb and judge whether
the file exists on the disk. If no, we find the TYPE. If FILE, return 1 means the file can be
rebuild. If NO_FILE, return -1 means the file can't be rebuild. If PIPE, return 0 means the
file exist. In fact, we don't process the FILE of PIPE.*/
int file_access(struct pnode_version pv)
{
   char                      *filetype, *filepath;
   char                      *path,*path1;

   filetype = get_string( pv.pnum, "TYPE" );
   printf("%s : ", filetype);
   if(filetype==NULL) return 0;
   switch ( *filetype ) {
   	case 'F':
		path = get_string( pv.pnum, "NAME");
		if( strncmp(".o.", path+strlen(path)-9, 3) ==0 ) {
			path1 = path+strlen(path)-7;
			*path1 = 0;
			//free(path1);
			}
		filepath = (char *)malloc( strlen(path) + 10 );
		sprintf(filepath, "/mnt/pass%s", path );//mount point
		free(path);
		if( access(filepath, 0)==0 ) {
			printf("%s, ---> exist.\n", filepath);
			free(filetype);free(filepath);
			return 0;
		}
		else {
			printf("%s, ---> NOT exist.\n", filepath);
			free(filetype);free(filepath);
			return 1;
		}
		break;
	case 'N':
		filepath = get_string( pv.pnum, "NAME");//do something about the files under directory /proc/pid/
		if( access(filepath, 0)==0 ) {
			printf("%s, ---> exist.\n", filepath);
			free(filetype);free(filepath);
			return 0;
		}
		else {
			printf("%s, ---> NOT exist.\n", filepath);
			free(filetype);free(filepath);
			return 0; //for test, set it return 0
		}
		break;
	case 'P': printf(" ---> exist.\n");free(filetype);return 0;
	default: free(filetype);return -1;
   }
}

/*return 1=delete,2=higher version,0=regular*/
int handling_state(struct pnode_version pv)
{
   DBC                  *parentc = NULL;
   DBT                   dbt_child;
   DBT                   dbt_parent;
   int                         ret;
   char                  * del,*name,*filepath,*bakfilepath;

   memset(&dbt_child, 0, sizeof(dbt_child));
   memset(&dbt_parent , 0, sizeof(dbt_parent));

	del = get_string(pv.pnum, "UNLINK");
	if(del==NULL) {
		ret = g_parentdb->db->cursor(g_parentdb->db, NULL, &parentc, 0);
		wdb_check_err(ret);
		pv.version+=1;
		dbt_child.data = &pv;
		dbt_child.size = sizeof(pv);
		ret = parentc->c_get(parentc, &dbt_child, &dbt_parent, DB_SET);
		if(ret == DB_NOTFOUND) return 0;
		else {
			name = get_string(pv.pnum,"NAME");
			filepath = (char *)malloc( strlen(name) + 10 );
			bakfilepath = (char *)malloc( strlen(filepath) + 5 );
			sprintf(filepath, "/mnt/pass%s", name );//mount point
			sprintf(bakfilepath, "%s.bak", filepath );
			rename(filepath, bakfilepath);
			free(name);free(filepath);free(bakfilepath);
			return 2;
		}
	}
	else {free(del);return 1;}
}


int analysis(struct pnode_version pv)
{
   DBC                  *childc = NULL;
   DBT                   dbt_child;
   DBT                   dbt_parent;
   provdb_key            child,parent;
   int                         ret,i=0,stat;

   memset(&dbt_child, 0, sizeof(dbt_child));
   memset(&dbt_parent , 0, sizeof(dbt_parent));
   memset(state , 0, sizeof(state));
   
   	ret = g_childdb->db->cursor(g_childdb->db, NULL, &childc, 0);
	wdb_check_err(ret);
	dbt_parent.data = &pv;
	dbt_parent.size = sizeof(pv);
	ret = childc->c_get(childc, &dbt_parent, &dbt_child, DB_SET);
   	while( ret != DB_NOTFOUND ) {
		memcpy(&child, dbt_child.data, dbt_child.size);
		memcpy(&parent, dbt_parent.data, dbt_parent.size);
		printf("Child: %llu.%u ",child.pnum,child.version);
		stat = handling_state(child);
		state[i] = stat;
		bak[i] = child;
		i++;
		ret = childc->c_get(childc, &dbt_parent, &dbt_child, DB_NEXT_DUP);
	}
	return i;
}


/*Return 1 means success, -1 means fail.*/
int proc_rebuild(pnode_t pnum)
{
   char                     *filepath;
   char                   **argv;
   char                   **env;
   char                     *pwd;
   int                          i, status;
   pid_t                      pid;

   filepath = get_string(pnum, "NAME");
   printf("Path : %s \n",filepath);
   
   argv = get_multistring(pnum, "ARGV");
   /*
   printf("Argv : ");
   for(i=0;;i++) if(argv[i]!=NULL) printf("\"%s\" ",argv[i]);else break;
   printf("\n");
*/
   env = get_multistring(pnum, "ENV");
   /*
   printf("Env : ");
   for(i=0;;i++) if(env[i]!=NULL) printf("\"%s\" ",env[i]);else break;
   printf("\n");
   */
   for(i=0;;i++) if( strncmp("PWD=", env[i], 4)==0 ) {pwd=env[i]+4;break;}

   pid = fork();
   if( pid == 0 ) {
	printf("CHILD : PWD = \"%s\"\n",pwd);
	printf("filepath:%s\n",filepath);
   	chdir(pwd);
   	execve(filepath, argv, NULL);
	_exit(5);
   }
   else {
   	pid = wait(&status);
	i=WEXITSTATUS(status);
	printf("PARENT : Child PROC PID = %d, exit status = %d\n",pid,i);
   }

   for(i=0;;i++) if(argv[i]!=NULL) free(argv[i]);else break;
   for(i=0;;i++) if(env[i]!=NULL) free(env[i]);else break;
   free(filepath);free(argv);free(env);
   return 1;
}

int recover(int num)
{
   int                        i,ret;
   char                  *name,*filepath,*bakfilepath;

	for(i=0;i<num;i++)
  	    switch ( state[i] ) {
    	     case 0:
		break;
   	     case 1:
		name = get_string(bak[i].pnum, "NAME");
		filepath = (char *)malloc( strlen(name) + 10 );
		sprintf(filepath, "/mnt/pass%s", name );
		remove(filepath);
		free(name);free(filepath);
		name =NULL;filepath=NULL;
		break;
  	     case 2:
		name = get_string(bak[i].pnum, "NAME");
		filepath = (char *)malloc( strlen(name) + 10 );
		bakfilepath = (char *)malloc( strlen(filepath) + 5 );
		sprintf(filepath, "/mnt/pass%s", name );
		sprintf(bakfilepath, "%s.bak", filepath );
		ret = rename(bakfilepath, filepath);
		free(name);free(filepath);free(bakfilepath);
		name =NULL;filepath=NULL;bakfilepath=NULL;
		break;
   	   }
	return 0;
}


int execute1(struct pnode_version pv)
{
   DBC                  *parentc = NULL;
   DBT                   dbt_child;
   DBT                   dbt_parent;
   provdb_key            child;
   provdb_key            parent;
   provdb_key            proc;
   int                         ret;

   memset(&dbt_child, 0, sizeof(dbt_child));
   memset(&dbt_parent , 0, sizeof(dbt_parent));
   memset(state , 0, sizeof(state));
   
   if( 1 == (ret = file_access(pv)) ) {
   	ret = g_parentdb->db->cursor(g_parentdb->db, NULL, &parentc, 0);
	wdb_check_err(ret);
	dbt_child.data = &pv;
	dbt_child.size = sizeof(pv);
	parentc->c_get(parentc, &dbt_child, &dbt_parent, DB_SET);
	memcpy(&child, dbt_parent.data, dbt_parent.size);
	proc = child;
	printf("For rebulid %llu.%u, we should run PROC:%llu.%u\n", pv.pnum,pv.version,proc.pnum,proc.version);
	child.version = 1;//begin with version 1.
	dbt_child.data = &child;
	dbt_child.size = sizeof(child);
	ret = parentc->c_get(parentc, &dbt_child, &dbt_parent, DB_SET);
   	while( ret != DB_NOTFOUND ) {
		memcpy(&parent, dbt_parent.data, dbt_parent.size);
		memcpy(&child, dbt_child.data, dbt_child.size);
		if( child.pnum != proc.pnum ||child.version > proc.pnum ) break;
		if(parent.version==0){ret = parentc->c_get(parentc, &dbt_child, &dbt_parent, DB_NEXT);
		continue;}
		printf("The PROC's INPUT is %llu.%u\n",parent.pnum,parent.version);
		ret = execute1(parent);//change to state handling funtion
		if( ret == -1 ) return ret;
		ret = parentc->c_get(parentc, &dbt_child, &dbt_parent, DB_NEXT);
	}
	printf("The PROC %llu.%u's information:\n",proc.pnum,proc.version);
	ret = proc_rebuild(proc.pnum);
	return ret;
   }
   else return ret;
}


pnode_version execute(struct pnode_version pv)
{
   DBC                  *parentc = NULL;
   DBT                   dbt_child;
   DBT                   dbt_parent;
   provdb_key            child;
   provdb_key            parent;
   provdb_key            proc;
   int                         ret;

   memset(&dbt_child, 0, sizeof(dbt_child));
   memset(&dbt_parent , 0, sizeof(dbt_parent));
   memset(state , 0, sizeof(state));
   
   if( 1 == (ret = file_access(pv)) ) {
   	ret = g_parentdb->db->cursor(g_parentdb->db, NULL, &parentc, 0);
	wdb_check_err(ret);
	dbt_child.data = &pv;
	dbt_child.size = sizeof(pv);
	parentc->c_get(parentc, &dbt_child, &dbt_parent, DB_SET);
	memcpy(&child, dbt_parent.data, dbt_parent.size);
	proc = child;
	printf("For rebulid %llu.%u, we should run PROC:%llu.%u\n", pv.pnum,pv.version,proc.pnum,proc.version);
	child.version = 1;//begin with version 1.
	dbt_child.data = &child;
	dbt_child.size = sizeof(child);
	ret = parentc->c_get(parentc, &dbt_child, &dbt_parent, DB_SET);
   	while( ret != DB_NOTFOUND ) {
		memcpy(&parent, dbt_parent.data, dbt_parent.size);
		memcpy(&child, dbt_child.data, dbt_child.size);
		if( child.pnum != proc.pnum ||child.version > proc.pnum ) break;
		if(parent.version==0){ret = parentc->c_get(parentc, &dbt_child, &dbt_parent, DB_NEXT);
		continue;}
		printf("The PROC's INPUT is %llu.%u\n",parent.pnum,parent.version);
		ret = execute1(parent);//change to state handling funtion
		if( ret == -1 ) return ;
		ret = parentc->c_get(parentc, &dbt_child, &dbt_parent, DB_NEXT);
	}
	//printf("The PROC %llu.%u's information:\n",proc.pnum,proc.version);
	//ret = proc_rebuild(proc.pnum);
	return proc;
   }
   else exit(0);
}

/*According to the pnum, find the newest version.*/
static version_t get_file_version( pnode_t pnum )
{
   DBC                  *parentc = NULL;
   DBT                   dbt_child;
   DBT                   dbt_parent;
   provdb_key            child;
   version_t                version=0;
   int                   ret;

   memset(&dbt_child, 0, sizeof(dbt_child));
   memset(&dbt_parent , 0, sizeof(dbt_parent));

   ret = g_parentdb->db->cursor(g_parentdb->db, NULL, &parentc, 0);
   wdb_check_err(ret);

   child.pnum = pnum;
   child.version = 1;
   dbt_child.data = &child;
   dbt_child.size = sizeof(child);
   ret =  parentc->c_get(parentc, &dbt_child, &dbt_parent, DB_SET);
   while ( ret != DB_NOTFOUND ) {
   	memcpy( &child, dbt_child.data, dbt_child.size );
   	if( child.pnum == pnum ) version = child.version;
	else break;
	ret = parentc->c_get(parentc, &dbt_child, &dbt_parent, DB_NEXT);
   }
   return version;
}

/*Input the filename and find the newest file pnum & version.*/
static int get_file_pv(char * file_name, struct pnode_version * pv)
{
   DBC                  *namec = NULL;
   DBT                   dbt_name;
   DBT                   dbt_pnode;
   pnode_t             pnum;
   char                  *type;
   int                     ret;

   memset(&dbt_name , 0, sizeof(dbt_name));
   memset(&dbt_pnode, 0, sizeof(dbt_pnode));

   ret = g_namedb->db->cursor(g_namedb->db, NULL, &namec, 0);
   wdb_check_err(ret);

   dbt_name.data = file_name;
   dbt_name.size = strlen(file_name);
   ret = namec->c_get(namec, &dbt_name, &dbt_pnode, DB_SET);
   while ( ret != DB_NOTFOUND ) {
   	memcpy(&pnum, dbt_pnode.data, dbt_pnode.size);
	type = get_string(pnum, "TYPE");
	if( strncmp(type, "FILE", 4)==0 && pnum > pv->pnum) {
		pv->version = get_file_version( pnum );
		pv->pnum = pnum;
	}
	free(type),type = NULL;
	ret = namec->c_get(namec, &dbt_name, &dbt_pnode, DB_NEXT_DUP);
   }
   if( pv->pnum == 0) return 0;
   return 1;
}

static char ** get_multistring(pnode_t pnum, const char * attr)
{
   DBT    dbt_tnum;
   DBT    dbt_token;
   char ** argv;
   char *   element;
   int         ntoks, i;
   tnum_t * toks;

   DBC                       *pdbc;
   DBT                        dbt_key;
   DBT                        dbt_val;
   provdb_key                 key;
   provdb_val                *val=NULL;
   const char                *attrname;
   char                      *data = NULL;
   size_t                     datalen=0;
   int                        serrno;
   int                        ret;

   memset(&dbt_key, 0, sizeof(dbt_key));
   memset(&dbt_val, 0, sizeof(dbt_val));

   ret = g_provdb->db->cursor(g_provdb->db, NULL, &pdbc, 0);
   wdb_check_err(ret);

   key.pnum = pnum;
   key.version = 0;
   dbt_key.data = &key;
   dbt_key.size = sizeof(key);
   ret =  pdbc->c_get(pdbc, &dbt_key, &dbt_val, DB_SET);
   while ( ret != DB_NOTFOUND ) {
   	val = malloc(dbt_val.size);
	if ( val == NULL ) {
	serrno = errno;
	fprintf( stderr, "Malloc of provdb record failed %s\n",
		 strerror(serrno) );
	errno = serrno;
	return 0;
	}
   	memcpy(val, dbt_val.data,dbt_val.size);
	attrname = PROVDB_VAL_ATTR(val);
	data = PROVDB_VAL_VALUE(val);
       datalen = PROVDB_VAL_VALUELEN(val);
	if( strncmp(attr, attrname, 3) ==0 ) break;
	else ret = pdbc->c_get(pdbc, &dbt_key, &dbt_val, DB_NEXT_DUP);
	free(val),val = NULL;
   }

   if ( data == NULL ) return NULL;
   
   ntoks = datalen / sizeof(tnum_t);
   toks = malloc(datalen);
   memcpy(toks, data, datalen);
   free(val),val = NULL;
   argv = (char **)malloc(sizeof(char *)*(ntoks+1));

   for(i=0; i<ntoks;i++) {
   	memset(&dbt_tnum, 0, sizeof(dbt_tnum));
	memset(&dbt_token, 0, sizeof(dbt_token));
	dbt_tnum.data = &toks[i];
	dbt_tnum.size = sizeof(toks[i]);
	ret = g_tnum2tokdb->db->get(g_tnum2tokdb->db, NULL,&dbt_tnum, &dbt_token,0);
	wdb_check_err(ret);
	element = (char *)malloc(dbt_token.size+1);
	memcpy(element, dbt_token.data, dbt_token.size);
	element[dbt_token.size] = '\0';
	argv[i] = element;
   }
   free(toks),toks = NULL;
   argv[i] = NULL;
   return argv;
}

static char * get_string(pnode_t pnum, const char * attr)
{
   DBC                       *pdbc = NULL;
   DBT                        dbt_key;
   DBT                        dbt_val;
   provdb_key                 key;
   provdb_val                *val;
   const char                *attrname;
   char                      *data, *attrval = NULL;
   size_t                     datalen;
   int                        serrno;
   int                        ret;
   struct waldo_db * tmp=g_provdb;
   memset(&dbt_key, 0, sizeof(dbt_key));
   memset(&dbt_val, 0, sizeof(dbt_val));

   if(j) tmp=g_jprovdb;
   ret = tmp->db->cursor(tmp->db, NULL, &pdbc, 0);
   wdb_check_err(ret);

   key.pnum = pnum;
   if(strcmp(attr,"INPUT")==0)
   	key.version=1;
   else
   	key.version = 0;
   
   dbt_key.data = &key;
   dbt_key.size = sizeof(key);
   ret =  pdbc->c_get(pdbc, &dbt_key, &dbt_val, DB_SET);
   while ( ret != DB_NOTFOUND ) {
   	val = malloc(dbt_val.size);
	if ( val == NULL ) {
	serrno = errno;
	fprintf( stderr, "Malloc of provdb record failed %s\n",
		 strerror(serrno) );
	errno = serrno;
	return 0;
	}
   	memcpy(val, dbt_val.data,dbt_val.size);
	attrname = PROVDB_VAL_ATTR(val);
	data = PROVDB_VAL_VALUE(val);
       datalen = PROVDB_VAL_VALUELEN(val);
	if( strncmp(attr, attrname, 4) ==0 || strncmp(attr,attrname,5)==0 ) {
		attrval = (char *)malloc(datalen+1);
		memcpy(attrval, data, datalen);
		attrval[datalen] = '\0';
		free(val),val = NULL;
		break;
	}
	else ret = pdbc->c_get(pdbc, &dbt_key, &dbt_val, DB_NEXT_DUP);
	free(val),val = NULL;
   }
   return attrval;
}
